#!/usr/bin/env python
#-*- coding: utf8 -*-
#
# This is a test suite for net module. Use unittest or Nose to run it.

import ipv4
import random
import unittest



class ConversionTest(unittest.TestCase):
    def runTest(self):
        self.failUnless(ipv4.Address.ston('0.0.0.0') == 0)
        self.failUnless(ipv4.Address.ston('255.255.255.255') == 0xffffffff)
        self.failUnlessRaises(ValueError, ipv4.Address.ston, '-1.0.0.0')
        self.failUnlessRaises(ValueError, ipv4.Address.ston, '800.0.300.0')
        self.failUnlessRaises(ValueError, ipv4.Address.ston, '-1.0.0.0')
        self.failUnlessRaises(ValueError, ipv4.Address.ston, '-200')
        self.failUnlessRaises(ValueError, ipv4.Address.ston, '800')
        self.failUnless(ipv4.Address.ntos(0) == '0.0.0.0')
        self.failUnless(ipv4.Address.ntos(0xffffffff) == '255.255.255.255')
        self.failUnlessRaises(ValueError, ipv4.Address.ntos, -1)
        self.failUnlessRaises(ValueError, ipv4.Address.ntos, 0xffffffffff)
    
    
    
class AddressObjectCreationTest(unittest.TestCase):
    def runTest(self):
        for (v, l) in generate_ips().items():
            self.failUnlessEqual(ipv4.Address(l).numeric, v, 'FAIL: %d' % (v,))
            self.failUnlessEqual(long(ipv4.Address(l)), v, 'FAIL: %d' % (v,))
            self.failUnlessEqual(int(ipv4.Address(l)), v, 'FAIL: %d' % (v,))
            self.failUnlessEqual(ipv4.Address(v).to_str(), l, 'FAIL: %s' % (l,))
            self.failUnlessEqual(str(ipv4.Address(v)), l, 'FAIL: %s' % (l,))



class AddressObjectArithmeticTest(unittest.TestCase):
    def runTest(self):
        self.failUnlessEqual(str(ipv4.Address('127.0.0.0') + 10), '127.0.0.10')
        self.failUnlessEqual(str(ipv4.Address('127.0.0.0') - 10), '126.255.255.246')
        try:
            ipv4.Address('0.0.0.10') - 20
            ipv4.Address('255.255.255.250') + 20
        except ValueError:
            pass
        else:
            self.fail()
            

            
class AddressObjectComparisonTest(unittest.TestCase):
    def runTest(self):
        for (v, l) in generate_ips().items():
            self.failUnless(ipv4.Address(v) == ipv4.Address(l), 'FAIL: {0} == {1}'.format(l, v))
        self.failUnless(ipv4.Address('127.0.0.1') < ipv4.Address('127.10.20.30'), 'FAIL: \'127.0.0.1\' < \'127.10.20.30\'')
        self.failUnless(ipv4.Address('127.10.20.30') > ipv4.Address('127.0.0.1'), 'FAIL: \'127.10.20.30\' > \'127.0.0.1\'')
        self.failUnless(ipv4.Address('127.10.20.30') != ipv4.Address('127.0.0.1'), 'FAIL: \'127.10.20.30\' != \'127.0.0.1\'')
        
        
        
class NetworkObjectCreationTest(unittest.TestCase):
    def runTest(self):
        self.failUnlessEqual(str(ipv4.Network('127.0.0.0/8')), '127.0.0.0 - 127.255.255.255', 'FAIL: Network(\'127.0.0.0/8\')')
        self.failUnlessEqual(str(ipv4.Network('127.1.1.0/24')), '127.1.1.0 - 127.1.1.255', 'FAIL: Network(\'127.1.1.0/24\')')
        self.failUnlessEqual(str(ipv4.Network('127.1.1.0/24')), '127.1.1.0 - 127.1.1.255', 'FAIL: Network(\'127.1.1.0/24\')')
        self.failUnlessEqual(str(ipv4.Network('127.0.0.0/19')), '127.0.0.0 - 127.0.31.255', 'FAIL: Network(\'127.0.0.0/19\')')
        
        self.failUnlessEqual(str(ipv4.Network('127.0.0.0 - 127.255.255.255')), '127.0.0.0 - 127.255.255.255', 'FAIL: Network(\'127.0.0.0 - 127.255.255.255\')')
        self.failUnlessEqual(str(ipv4.Network('127.1.1.0 - 127.1.1.255')), '127.1.1.0 - 127.1.1.255', 'FAIL: Network(\'127.1.1.0 - 127.1.1.255\')')
        self.failUnlessEqual(str(ipv4.Network('127.1.1.0 - 127.1.1.255')), '127.1.1.0 - 127.1.1.255', 'FAIL: Network(\'127.1.1.0 - 127.1.1.255\')')
        self.failUnlessEqual(str(ipv4.Network('127.0.0.0 - 127.0.31.255')), '127.0.0.0 - 127.0.31.255', 'FAIL: Network(\'127.0.0.0 - 127.0.31.255\')')
        
        self.failUnlessEqual(str(ipv4.Network((ipv4.Address('127.0.0.0'), ipv4.Address('127.255.255.255')))), '127.0.0.0 - 127.255.255.255', 'FAIL: Network((Address(\'127.0.0.0\'), Address(\'127.255.255.255\')))')
        self.failUnlessEqual(str(ipv4.Network((ipv4.Address('127.1.1.0'), ipv4.Address('127.1.1.255')))), '127.1.1.0 - 127.1.1.255', 'FAIL: Network((Address(\'127.1.1.0\'), Address(\'127.1.1.255\')))')
        self.failUnlessEqual(str(ipv4.Network((ipv4.Address('127.1.1.0'), ipv4.Address('127.1.1.255')))), '127.1.1.0 - 127.1.1.255', 'FAIL: Network((Address(\'127.1.1.0\'), Address(\'127.1.1.255\')))')
        self.failUnlessEqual(str(ipv4.Network((ipv4.Address('127.0.0.0'), ipv4.Address('127.0.31.255')))), '127.0.0.0 - 127.0.31.255', 'FAIL: Network((Address(\'127.0.0.0\'), Address(\'127.0.31.255\')))')
        
        self.failUnlessRaises(ValueError, ipv4.Network, '127.10.20.30 - 127.0.0.1')
        self.failUnlessRaises(ValueError, ipv4.Network, '127.10.20.30/300')
        self.failUnlessRaises(ValueError, ipv4.Network, '127.10.20.30/-3')
        self.failUnlessRaises(ValueError, ipv4.Network, '127.10.20.30/')
        
        
        
class NetworkObjectMaskCalculationTest(unittest.TestCase):
    def runTest(self):
        self.failUnlessEqual(ipv4.Network('127.0.0.0 - 127.255.255.255').mask, 8)
        self.failUnlessEqual(ipv4.Network('127.0.0.0/8').mask, 8)
        self.failUnlessEqual(ipv4.Network('127.0.0.0 - 127.0.31.255').mask, 19)
        self.failUnlessEqual(ipv4.Network('127.0.0.0/19').mask, 19)
        
        
        
class NetworkObjectComparisonTest(unittest.TestCase):
    def runTest(self):
        self.failUnless(ipv4.Network('127.0.0.0 - 127.255.255.255') == ipv4.Network('127.0.0.0/8'))
        self.failUnless(ipv4.Network('127.0.0.0 - 127.0.31.255') == ipv4.Network('127.0.0.0/19'))
        self.failUnless(ipv4.Network('127.0.0.0 - 127.0.31.255') == ipv4.Network((ipv4.Address('127.0.0.0'), ipv4.Address('127.0.31.255'))))
        self.failUnless(ipv4.Network('127.0.0.0 - 127.0.31.255') != ipv4.Network('128.0.0.0/19'))
        self.failUnless(ipv4.Network('127.0.0.0 - 127.0.31.255') != ipv4.Network('127.0.0.0/12'))
        
        self.failUnless(ipv4.Network('127.0.0.0 - 127.0.1.0') in ipv4.Network('127.0.0.0/8'))
        self.failUnless(ipv4.Network('127.0.0.0 - 127.0.1.0') not in ipv4.Network('128.0.0.0/8'))
        
        
        
class NetworkObjectArithmeticTest(unittest.TestCase):
    def runTest(self):
        self.failUnless(ipv4.Network('127.0.0.0 - 127.255.255.255') + 10 == ipv4.Network('127.0.0.0 - 127.0.0.10'))
        self.failUnless(ipv4.Network('127.0.0.0 - 127.255.255.255') - 200 == ipv4.Network('127.0.0.0 - 127.255.255.55'))
        
        
        
class NetworkObjectSlicingTest(unittest.TestCase):
    def runTest(self):
        self.failUnless(ipv4.Network('192.168.0.0/24')[10] == ipv4.Address('192.168.0.10'))
        self.failUnless(ipv4.Network('192.168.0.0/24')[-10] == ipv4.Address('192.168.0.245'))
        self.failUnless(ipv4.Network('192.168.0.0/24')[10:11] == ipv4.Network('192.168.0.10 - 192.168.0.11'))
        self.failUnless(ipv4.Network('192.168.0.0/24')[-10:11] == ipv4.Network('192.168.0.11 - 192.168.0.245'))
        self.failUnless(ipv4.Network('192.168.0.0/24')[10:-11] == ipv4.Network('192.168.0.10 - 192.168.0.244'))
        self.failUnless(ipv4.Network('192.168.0.0/24')[-10:-11] == ipv4.Network('192.168.0.244 - 192.168.0.245'))
        
    
    
def generate_ips(size=100):
    ips = {}
    while len(ips) < size:
        v = random.randrange(0xffffffff)
        ips[v] = ipv4.Address.ntos(v)
    return ips



def suite_Address():
    return unittest.TestSuite((ConversionTest,
                               AddressObjectCreationTest,
                               AddressObjectArithmeticTest,
                               AddressObjectComparisonTest))


def suite_Network():
    return unittest.TestSuite((NetworkObjectCreationTest,
                               NetworkObjectMaskCalculationTest,
                               NetworkObjectComparisonTest,
                               NetworkObjectArithmeticTest,
                               NetworkObjectSlicingTest))


def suite():
    return unittest.TestSuite((suite_Address(), suite_Network()))
